from arm64_tester.subroutines.array_subroutine import ArraySubroutine
from arm64_tester.subroutines.mixed_subroutine import MixedSubroutine
from arm64_tester.subroutines.numeric_subroutine import NumericSubroutine
from arm64_tester.subroutines.void_subroutine import VoidSubroutine
